/*
IOT Sensor Board Amazon Web Services (AWS) Demo with configuration webpage
for ESP32 Pico Kit
Elektor / Jens Nickel
My Journey in the IoT
*/

#include <WiFi.h>

#include <Preferences.h>   // this library is used to get access to Non-volatile storage (NVS) of ESP32
// see https://github.com/espressif/arduino-esp32/blob/master/libraries/Preferences/examples/StartCounter/StartCounter.ino





// +++++++++++++++++++ Start of Webserver Library +++++++++++++++++++

// as ESP32 Arduino Version

WiFiClient myclient;
WiFiServer server(80);


void Webserver_Start()
{
  server.begin();     // Start TCP/IP-Server on ESP32
}



String WebRequestHostAddress;     // global variable used to store Server IP-Address of HTTP-Request


//  Call this function regularly to look for client requests
//  template see https://github.com/espressif/arduino-esp32/blob/master/libraries/WiFi/examples/SimpleWiFiServer/SimpleWiFiServer.ino
//  returns empty string if no request from any client
//  returns GET Parameter: everything after the "/?" if ADDRESS/?xxxx was entered by the user in the webbrowser
//  returns "-" if ADDRESS but no GET Parameter was entered by the user in the webbrowser
//  remark: client connection stays open after return
String Webserver_GetRequestGETParameter()
{
  String GETParameter = "";
  
  myclient = server.available();   // listen for incoming clients

  //Serial.print(".");
  
  if (myclient) {                            // if you get a client,
    Serial.println("New Client.");           // print a message out the serial port
    String currentLine = "";                 // make a String to hold incoming data from the client
    
    while (myclient.connected()) {           // loop while the client's connected
      
      if (myclient.available()) {            // if there's bytes to read from the client,
        
        char c = myclient.read();            // read a byte, then
        Serial.write(c);                     // print it out the serial monitor

        if (c == '\n') {                     // if the byte is a newline character

          // if the current line is blank, you got two newline characters in a row.
          // that's the end of the client HTTP request
          if (currentLine.length() == 0) {
            
            if (GETParameter == "") {GETParameter = "-";};    // if no "GET /?" was found so far in the request bytes, return "-"
            
            // break out of the while loop:
            break;
        
          } else {    // if you got a newline, then clear currentLine:
            currentLine = "";
          }
          
        } else if (c != '\r') {  // if you got anything else but a carriage return character,
          currentLine += c;      // add it to the end of the currentLine
        }

        if (c=='\r' && currentLine.startsWith("GET /?")) 
        // we see a "GET /?" in the HTTP data of the client request
        // user entered ADDRESS/?xxxx in webbrowser, xxxx = GET Parameter
        {
          
          GETParameter = currentLine.substring(currentLine.indexOf('?') + 1, currentLine.indexOf(' ', 6));    // extract everything behind the ? and before a space
                    
        }


        if (c=='\r' && currentLine.startsWith("Host:")) 
        // we see a "Host:" in the HTTP data of the client request
        // user entered ADDRESS or ADDRESS/... in webbrowser, ADDRESS = Server IP-Address of HTTP-Request
        {
          int IndexOfBlank = currentLine.indexOf(' ');
          WebRequestHostAddress = currentLine.substring(IndexOfBlank + 1, currentLine.length());    // extract everything behind the space character and store Server IP-Address of HTTP-Request
          
        }
        
      }
      
    }
    
  }

  return GETParameter;
}



// Send HTML page to client, as HTTP response
// client connection must be open (call Webserver_GetRequestGETParameter() first)
void Webserver_SendHTMLPage(String HTMLPage)
{
   String httpResponse = "";

   // begin with HTTP response header
   httpResponse += "HTTP/1.1 200 OK\r\n";
   httpResponse += "Content-type:text/html\r\n\r\n";

   // then the HTML page
   httpResponse += HTMLPage;

   // The HTTP response ends with a blank line:
   httpResponse += "\r\n";
    
   // send it out to TCP/IP client = webbrowser 
   myclient.println(httpResponse);

   // close the connection
   myclient.stop();
    
   Serial.println("Client Disconnected.");
   
};



// +++++++++++++++++++ End of Webserver library +++++++++++++++++++++

















#define PIN_LED_BOARD  15
#define PIN_ACTOR      15   // the digital output we want to control

#define PIN_LED_GREEN  27   // pin numbers connected to single LEDs of RGB LED
#define PIN_LED_RED    26
#define PIN_LED_BLUE   25

#define PIN_SENSOR     36   // the analog input for sensor

#define LED_ON    128
#define LED_OFF   0

#define ACTOR_ON    1
#define ACTOR_OFF   0



// Switch Actor Output
// ActorState must be ACTOR_OFF or ACTOR_ON
void SwitchActor(byte ActorState)
{
    if (ActorState == ACTOR_OFF)
    {
      digitalWrite(PIN_ACTOR, LOW);
    }
    else
    {      
      digitalWrite(PIN_ACTOR, HIGH);
    }
}


// Switch LED on the board
// LEDState must be LED_OFF or LED_ON
void SwitchBoardLED(byte LEDState)
{
    if (LEDState == LED_OFF)
    {
      digitalWrite(PIN_LED_BOARD, LOW);
    }
    else
    {      
      digitalWrite(PIN_LED_BOARD, HIGH);
    }
}







// this section handles configuration values which can be configured via webpage form in a webbrowser

// 8 configuration values max
String ConfigName[8];     // name of the configuration value
String ConfigValue[8];    // the value itself (String)
int    ConfigStatus[8];   // status of the value    0 = not set    1 = valid   -1 = not valid
int    ConfigType[8];     // type of the value    0 = not set    1 = String (textbox)   2 = Byte (slider)


// Initalize the values 
void InitializeConfigValues()
{
  for (int count = 0; count < 8; count++)
  {
    ConfigName[count] = "";
    ConfigValue[count] = "";
    ConfigStatus[count] = 0;
    ConfigType[count] = 0;
  }
}


// Build a HTML page with a form which shows textboxes to enter the values
// returns the HTML code of the page
String EncodeFormHTMLFromValues(String TitleOfForm, int CountOfConfigValues)
{
   // Head of the HTML page
   String HTMLPage = "<!DOCTYPE html><html><body><h2>" + TitleOfForm + "</h2><form><table>";

   // for each configuration value
   for (int c = 0; c < CountOfConfigValues; c++)
   {
      // set background color by the status of the configuration value
      String StyleHTML = "";
      if (ConfigStatus[c] == 0) { StyleHTML = " Style =\"background-color: #FFE4B5;\" " ;};   // yellow
      if (ConfigStatus[c] == 1) { StyleHTML = " Style =\"background-color: #98FB98;\" " ;};   // green
      if (ConfigStatus[c] == -1) { StyleHTML = " Style =\"background-color: #FA8072;\" " ;};  // red

      String TableRowHTML = "";
  
      if (ConfigType[c] == 1)    //  string value, entered in text box
      {
        // build the HTML code for a table row with configuration value name and the value itself inside a textbox   
        TableRowHTML = "<tr><th>" + ConfigName[c] + "</th><th><input name=\"" + ConfigName[c] + "\" value=\"" + ConfigValue[c] + "\" " + StyleHTML + " /></th></tr>";
      }
  
      if (ConfigType[c] == 2)
      {
        // build the HTML code for a table row with configuration value name and the byte value represented by a slider 
        TableRowHTML = "<tr><th>" + ConfigName[c] + "</th><th><input type=\"range\" min=\"1\"  max=\"255\" name=\"" + ConfigName[c] + "\" value=\"" + ConfigValue[c] + "\" " + StyleHTML + " /></th></tr>";
      }
      
      // add the table row HTML code to the page
      HTMLPage += TableRowHTML;
   }

   // add the submit button
   HTMLPage += "</table><br/><input type=\"submit\" value=\"Submit\" />";

   // footer of the webpage
   HTMLPage += "</form></body></html>";
   
   return HTMLPage;
}


// decodes Espace-Codes in URL strings / GET parameters
String DecodeURLChars(String strURL)
{
  strURL.replace("%24", "$");
  strURL.replace("%2F", "/");
  strURL.replace("%3A", ":");
  strURL.replace("%23", "#");
  strURL.replace("%7B", "{");
  strURL.replace("%7D", "}");
  strURL.replace("%22", String(char('\"')));
  return strURL;
}


// Decodes a GET parameter (expression after ? in URI (URI = expression entered in address field of webbrowser)), like "Country=Germany&City=Aachen"
// and set the ConfigValues
int DecodeGETParameterAndSetConfigValues(String GETParameter)
{
   
   int posFirstCharToSearch = 1;
   int count = 0;
   
   // while a "&" is in the expression, after a start position to search
   while (GETParameter.indexOf('&', posFirstCharToSearch) > -1)
   {
     int posOfSeparatorChar = GETParameter.indexOf('&', posFirstCharToSearch);  // position of & after start position
     int posOfValueChar = GETParameter.indexOf('=', posFirstCharToSearch);      // position of = after start position
  
     String strChars = GETParameter.substring(posOfValueChar + 1, posOfSeparatorChar);  // extract everything between = and & and enter it in the ConfigValue
     
     ConfigValue[count] = DecodeURLChars(strChars);  //  Decode Escape characters 

     posFirstCharToSearch = posOfSeparatorChar + 1;  // shift the start position to search after the &-char 
     count++;
   }

   // no more & chars found
   
   int posOfValueChar = GETParameter.indexOf('=', posFirstCharToSearch);       // search for =
   
   String strChars = GETParameter.substring(posOfValueChar + 1, GETParameter.length());  // extract everything between = and end of string
   
   ConfigValue[count] = DecodeURLChars(strChars);  //  Decode Escape characters 
   
   count++;

   return count;  // number of values found in GET parameter
}








// +++++++++++++++++++ Start of Application Code +++++++++++++++++++

// include RGB LED library
#include <RGBLED.h>

// construct the object BreadboardRGBLED of class type RGBLED
RGBLED BreadboardRGBLED(PIN_LED_RED, PIN_LED_GREEN, PIN_LED_BLUE);


// include WiFi Network library
#include <WiFiNetwork.h>

// construct the object WiFiNetwork of class WiFiNetwork
WiFiNetwork myWiFiNetwork = WiFiNetwork();


// include TCP Client library
#include <TCPClient.h>

// construct the object mqttTCPClient of class TCPClient
TCPClient mqttTCPClient = TCPClient();


// include MQTT Client for TLS library
#include <MQTTClient.h>

// construct the object myMQTTClient of class MQTTClient
MQTTClient myMQTTClient = MQTTClient();

Preferences preferences;   // we must generate this object of the preference library

#define NODESTATE_RESET                    3    // S3 = connection + subscription established after user reset
#define NODESTATE_PING                     2    // S2 = connection still open, successful ping
#define NODESTATE_OK                       1    // S1 = connection + subscription established
#define NODESTATE_NOTSPECIFIEDERROR        0    // E0 = unspecified error
#define NODESTATE_NOTCPCONNECTION          -1   // E1 = connection not established because of no tcp connection to test broker
#define NODESTATE_NOMQTTCONNECTION         -2   // E2 = connection not established because broker rejected MQTT connection attempt
#define NODESTATE_NOMQTTSUBSCRIPTION       -3   // E3 = broker rejected subscription attempt
#define NODESTATE_NOPING                   -4   // E4 = ping was not successful, connection lost



#define NETWORKSTATE_LOGINREQUESTED       2
#define NETWORKSTATE_LOGGED               1
#define NETWORKSTATE_NOTLOGGED            0

byte RouterNetworkDeviceState; 

byte MQTTClient_Connected;
byte CounterForMQTT;
byte CounterForPing;
byte CounterForLoginRouterNetworkAttempt;



void setup() 
{
  
  myMQTTClient.myTCPClient = mqttTCPClient;    // tell the MQTT object the TCP object because MQTT library is using the TCP/IP library too
  
  pinMode(PIN_LED_BOARD, OUTPUT);
  pinMode(PIN_ACTOR, OUTPUT);

  // Switch the RGB LED to yellow (with a public function of our RGBLED object)
  BreadboardRGBLED.SwitchRGBLED(LED_YELLOW); 

  // switch LED off
  SwitchActor(ACTOR_OFF);
  
  // Open serial communications and wait for port to open
  Serial.begin(19200);
  Serial.println("Hello ESP32!");

  // put the ESP32 in both modes
  myWiFiNetwork.SetBothModesNetworkStationAndAccessPoint();

  // Start access point with SSID "ESP32_MyJourneyIoT"
  int success2 = myWiFiNetwork.AccessPointStart("ESP32_MyJourneyIoT");

  Serial.println(myWiFiNetwork.GetIPAddressInOwnNetwork());

  // start the webserver to listen for request of clients (in LAN or own ESP32 network)
  Webserver_Start();


  // initialize config values and set names
  InitializeConfigValues();
  ConfigName[0] = "LED";
  ConfigType[0] = 1;     // type textbox
  
  ConfigName[1] = "SSID";
  ConfigType[1] = 1;
  
  ConfigName[2] = "PASSWORD";
  ConfigType[2] = 1;
  
  ConfigName[3] = "BROKER ADDRESS";
  ConfigType[3] = 1;

  ConfigName[4] = "CLIENT/DEVICE ID";
  ConfigType[4] = 1; 

  ConfigName[5] = "SENSOR ID/TOKEN";
  ConfigType[5] = 1;         

  ConfigName[6] = "TOPIC";
  ConfigType[6] = 1;         

  ConfigName[7] = "PAYLOAD";
  ConfigType[7] = 1;         


  preferences.begin("MyJourney", false);   // see https://github.com/espressif/arduino-esp32/blob/master/libraries/Preferences/examples/StartCounter/StartCounter.ino 

  // takeout 7 Strings out of the Non-volatile storage
  String strSSID = preferences.getString("SSID", "");
  String strPassword = preferences.getString("Password", "");
  String strBrokerAddress = preferences.getString("BrokerAddress", "");
  String strDeviceID = preferences.getString("DeviceID", "");
  String strSensorID = preferences.getString("SensorID", "");
  String strTopic = preferences.getString("Topic", "");
  String strPayload = preferences.getString("Payload", "");
  
  // put the NVS stored values in RAM for the program
  ConfigValue[1] = strSSID;
  ConfigValue[2] = strPassword;
  ConfigValue[3] = strBrokerAddress;
  ConfigValue[4] = strDeviceID;
  ConfigValue[5] = strSensorID;
  ConfigValue[6] = strTopic;
  ConfigValue[7] = strPayload;
 
  // initialize state variables and counters
  MQTTClient_Connected = false;
  CounterForMQTT = 0;
  CounterForPing = 0;
  
  RouterNetworkDeviceState = NETWORKSTATE_NOTLOGGED;  // set the state
  CounterForLoginRouterNetworkAttempt = 40;    
  
}



// check the ConfigValues and set ConfigStatus
// process the first ConfigValue to switch something
void ProcessAndValidateConfigValues(int countValues)
{
  if (countValues > 8) {countValues = 8;};

  // store the second to eigth value in non-volatile storage
  if (countValues > 4)
  {
    preferences.putString("SSID",          ConfigValue[1]);
    preferences.putString("Password",      ConfigValue[2]);
    preferences.putString("BrokerAddress", ConfigValue[3]);
    preferences.putString("DeviceID",      ConfigValue[4]);
    preferences.putString("SensorID",      ConfigValue[5]);
    preferences.putString("Topic",         ConfigValue[6]);
    preferences.putString("Payload",       ConfigValue[7]);
  }

  // in our application the first value must be "00" or "FF" (as text string)
  if ((ConfigValue[0].equals("00")) || (ConfigValue[0].equals("FF")))
  {
    ConfigStatus[0] = 1;    // Value is valid
  }
  else
  {
    ConfigStatus[0] = -1;   // Value is not valid
  }

  // first config value is used to switch LED ( = Actor)
  if (ConfigValue[0].equals("00"))   
  {
    SwitchActor(ACTOR_OFF);
  }
  
  if (ConfigValue[0].equals("FF"))
  {
    SwitchActor(ACTOR_ON);
  }

  
}



// Read incoming request from client (if there is any)
// process values in GET parameter
// and send out new HTML page to client
// returns 1 if GET parameter with new values was received, after URL address 192.168.4.1 (as a sign that user wants to configure device once new)
// returns 0 if any other case 
int GetConfigRequestAndSendHTMLPage()
{
    int ConfigRequestReceived = 0;
    
    String GETParameter = Webserver_GetRequestGETParameter();   // look for client request
  
    if (GETParameter.length() > 0)        // we got a request, client connection stays open
    {
      if (GETParameter.length() > 1)      // request contains some GET parameter
      {
          
          int countValues = DecodeGETParameterAndSetConfigValues(GETParameter);     // decode the GET parameter and set ConfigValues
  
          if (WebRequestHostAddress == "192.168.4.1")                               // the user entered this address in browser, with GET parameter values for configuration
          {
    
                ProcessAndValidateConfigValues(8);                                  // check and process 5 ConfigValues, switch the LED, store SSID, Password, DeviceID and BrokerAddress in non-volatile storage
            
                ConfigRequestReceived = 1;
          }
          
      }

      String HTMLPage;
       
      if (WebRequestHostAddress == "192.168.4.1")                  //   the user entered this address in the browser, to get the configuration webpage               
      {
          HTMLPage = EncodeFormHTMLFromValues("ESP32 Webserver CONFIG", 8) + "<br>IP Address: " + myWiFiNetwork.GetOwnIPAddressInRouterNetwork();       // build a new Configuration webpage with form and new ConfigValues entered in textboxes or represented by sliders
      }
      
      Webserver_SendHTMLPage(HTMLPage);    // send out the webpage to client = webbrowser and close client connection
      
  }

  return ConfigRequestReceived;
}



// try to login to router network
// and set RGB LED
void LoginRouterNetworkAndSetRGB()
{
  String strSSID = preferences.getString("SSID", "");                 // takeout SSID and Password out of non-volatile storage  
  String strPassword = preferences.getString("Password", "");              

  char* txtSSID = const_cast<char*>(strSSID.c_str());                 // convert to char*
  char* txtPassword = const_cast<char*>(strPassword.c_str());         // https://coderwall.com/p/zfmwsg/arduino-string-to-char

  int successDisconnect = myWiFiNetwork.RouterNetworkDisconnect();             // disconnect from router network

  BreadboardRGBLED.SwitchRGBLED(LED_YELLOW); 

  int successConnect = myWiFiNetwork.RouterNetworkConnect(txtSSID, txtPassword);   // then try to connect once new with new login-data

  // print out local address of ESP32 in Router network (LAN)
  Serial.println(myWiFiNetwork.GetOwnIPAddressInRouterNetwork());
    
  if (successConnect == 1)
  {
      BreadboardRGBLED.SwitchRGBLED(LED_YELLOW);                      // switch the RGB LED color to yellow
      BreadboardRGBLED.setRGB(255, 255, 0);                           // and set the default color to yellow for dimming
      RouterNetworkDeviceState = NETWORKSTATE_LOGGED;                 // set the state

      CounterForMQTT = 0;
  }
  else
  {
      BreadboardRGBLED.SwitchRGBLED(LED_RED);                         // switch RGB LED to red 
      RouterNetworkDeviceState = NETWORKSTATE_NOTLOGGED;              // set the state                 
  }   

  // we are just logged in in network, so no MQTT connection
  MQTTClient_Connected = false;
  
}




// Connect to MQTT Broker and switch RGB-LEDs
int ConnectToMQTTBroker()
{   
    int NodeState;

    // closes TCP connection
    mqttTCPClient.Close();

    // get the broker address out of non-volatile storage
    String strBrokerAddress = preferences.getString("BrokerAddress", "");
    char* txtBrokerAddress = const_cast<char*>(strBrokerAddress.c_str());                 // convert to char*
       
    // TCP-connect to MQTT Broker TCP Server
    int TCP_Connect_Result = mqttTCPClient.Connect(txtBrokerAddress, 1883);
    
    // Success?
    if (TCP_Connect_Result != 1)
    {
      // no = red LED
      BreadboardRGBLED.SwitchRGBLED(LED_RED);
      BreadboardRGBLED.setRGB(255, 0, 0);          // and set the default color to red for dimming 
      NodeState = NODESTATE_NOTCPCONNECTION;  
    }
    else
    {
      // yes = yellow LED
      BreadboardRGBLED.SwitchRGBLED(LED_YELLOW); 

      // take Device Name out of non-volatile storage
      String strDeviceID = preferences.getString("DeviceID", "");
      
      // MQTT CONNECT to MQTT broker , use Device ID as Client ID
      int MQTT_Connect_Result = myMQTTClient.Connect(strDeviceID, "", "");
      
      if (MQTT_Connect_Result == 1)
      {
        // connected
        BreadboardRGBLED.SwitchRGBLED(LED_GREEN);    // green LED
        BreadboardRGBLED.setRGB(0, 255, 0);          // and set the default color to green for dimming
        NodeState = NODESTATE_OK;

        Serial.println("MQTT connected.");        
      }
      else
      {
        // not connected
        BreadboardRGBLED.SwitchRGBLED(LED_RED);      // red LED
        BreadboardRGBLED.setRGB(255, 0, 0);          // and set the default color to red for dimming
        NodeState = NODESTATE_NOMQTTCONNECTION;  

        Serial.println("MQTT not connected.");
      }
   }

   return NodeState;
}



// Publish a MQTT message with SensorDate (here: 1 Byte) to MQTT Broker
void PublishSensorDate(String SensorName, byte SensorDate)
{
    // Get Configuration Parameters from storage
    
    String strKey = preferences.getString("Payload", "");
    String strDeviceID = preferences.getString("DeviceID", "");
    String strSensorID = preferences.getString("SensorID", "");

    // replace code expression in Payload Parameter
    strKey.replace("$D$", strDeviceID);
    strKey.replace("$S$", strSensorID);
    strKey.replace("$V$", String(SensorDate));
    strKey.replace("$N$", SensorName);
    strKey.replace("#", String(char('\"')));
    
    String MString = strKey;

    Serial.println(MString);
  
    int Slength = MString.length();

    // copy the bytes of the payload string in the payload buffer
    for (int j = 0; j < Slength; j++)
    {
        myMQTTClient.PayloadBuffer[j] = MString[j];
    }

    //  take Device Name out of non-volatile storage
    String strTopic = preferences.getString("Topic", "");

    // replace code expression in Topic Parameter
    strTopic.replace("$D$", strDeviceID);
    strTopic.replace("$S$", strSensorID);
    strTopic.replace("$N$", SensorName);

    String SensorTopic = strTopic;

    Serial.println(SensorTopic);

    // Publish the message with topic and payload
    myMQTTClient.Publish(SensorTopic, myMQTTClient.PayloadBuffer, Slength);

}




int CurrentSensorDate = 10;

// Send out light data via MQTT to MQTT Broker
// light data is sampled via ADC
void GetAndSendSensorData()
{
    // generate virtual sensor data (10..40)
    // not used here
    CurrentSensorDate++;
    if (CurrentSensorDate > 40) {CurrentSensorDate = 10;};
    
    // read the voltage at pin 36, 0..4096, divide by 64, means 0..63
    int analog_value = analogRead(PIN_SENSOR) >> 6;

    // Send sensor data via MQTT
    PublishSensorDate("light", analog_value); 

    Serial.print("VAL:");
    Serial.println(analog_value);      
}






void loop() {

   // check if request from client was received (PC or smartphone browser with HTML config form)
   int ConfigRequestReceived = GetConfigRequestAndSendHTMLPage();
   if (ConfigRequestReceived == 1)  // yes
   {
      RouterNetworkDeviceState = NETWORKSTATE_LOGINREQUESTED;    // set state variable that a new login attempt follows
      CounterForLoginRouterNetworkAttempt = 40;                  // immidiately
   }
    
   if (RouterNetworkDeviceState != NETWORKSTATE_LOGGED)    // if we are not logged in or login attempt requested by user
   {
      if (CounterForLoginRouterNetworkAttempt > 40)        // counter for a delay between login attempts (approx. every 2 s)
      {
        LoginRouterNetworkAndSetRGB();                     // login attempt
        CounterForLoginRouterNetworkAttempt = 0;
      }
      CounterForLoginRouterNetworkAttempt++;
   }
   
   if (RouterNetworkDeviceState == NETWORKSTATE_LOGGED)    // if we are logged in
   {
     BreadboardRGBLED.dimRGB();                            // process a dimming step

     CounterForMQTT++;
     if (CounterForMQTT > 40)                              // do following approx every 2 seconds
     {
       CounterForMQTT = 0;

       CounterForPing++;
       if (CounterForPing > 4)                             // do following approx every 10 seconds
       {
          CounterForPing = 0;

          int PingResult = myMQTTClient.Ping();           // Send out Ping to MQTT broker
          if (PingResult == 0)                             // no Ping response
          {
            MQTTClient_Connected = false;                  // set state variable that we need a new MQTT connection
          }
          
       }
              
       if (MQTTClient_Connected == false)                  // no MQTT connection
       {
          int NodeState = ConnectToMQTTBroker();                  // try to connect to AWS MQTT Broker and switch RGB-LEDs
    
          if (NodeState > 0)                               // MQTT connection established successfully
          {
            MQTTClient_Connected = true;                   // state variable is set that we can go in normal mode and send out sensor data
          }
          else                                             // MQTT connection could not be established
          {
            RouterNetworkDeviceState = NETWORKSTATE_NOTLOGGED;   // we assume that Wifi access was lost
            CounterForLoginRouterNetworkAttempt = 0;             // new Wifi login attempt with 2 s delay
          }
       }
    
       if (MQTTClient_Connected == true)                   // we are successfully connected with MQTT broker
       {
          GetAndSendSensorData();        
       }
    }
  }
  delay(50);          // wait 50 ms
}
