#include "TCPClientSecure.h"

// TCP Library for secure TLS communication
// as ESP32 Arduino Version


#include "AWS_Certificate.h"


/* root CA can be download here:
 *  https://www.symantec.com/content/en/us/enterprise/verisign/roots/VeriSign-Class%203-Public-Primary-Certification-Authority-G5.pem
 **/
char* rootCABuff = \
"-----BEGIN CERTIFICATE-----\n" \
"MIIE0zCCA7ugAwIBAgIQGNrRniZ96LtKIVjNzGs7SjANBgkqhkiG9w0BAQUFADCB\n" \
"yjELMAkGA1UEBhMCVVMxFzAVBgNVBAoTDlZlcmlTaWduLCBJbmMuMR8wHQYDVQQL\n" \
"ExZWZXJpU2lnbiBUcnVzdCBOZXR3b3JrMTowOAYDVQQLEzEoYykgMjAwNiBWZXJp\n" \
"U2lnbiwgSW5jLiAtIEZvciBhdXRob3JpemVkIHVzZSBvbmx5MUUwQwYDVQQDEzxW\n" \
"ZXJpU2lnbiBDbGFzcyAzIFB1YmxpYyBQcmltYXJ5IENlcnRpZmljYXRpb24gQXV0\n" \
"aG9yaXR5IC0gRzUwHhcNMDYxMTA4MDAwMDAwWhcNMzYwNzE2MjM1OTU5WjCByjEL\n" \
"MAkGA1UEBhMCVVMxFzAVBgNVBAoTDlZlcmlTaWduLCBJbmMuMR8wHQYDVQQLExZW\n" \
"ZXJpU2lnbiBUcnVzdCBOZXR3b3JrMTowOAYDVQQLEzEoYykgMjAwNiBWZXJpU2ln\n" \
"biwgSW5jLiAtIEZvciBhdXRob3JpemVkIHVzZSBvbmx5MUUwQwYDVQQDEzxWZXJp\n" \
"U2lnbiBDbGFzcyAzIFB1YmxpYyBQcmltYXJ5IENlcnRpZmljYXRpb24gQXV0aG9y\n" \
"aXR5IC0gRzUwggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQCvJAgIKXo1\n" \
"nmAMqudLO07cfLw8RRy7K+D+KQL5VwijZIUVJ/XxrcgxiV0i6CqqpkKzj/i5Vbex\n" \
"t0uz/o9+B1fs70PbZmIVYc9gDaTY3vjgw2IIPVQT60nKWVSFJuUrjxuf6/WhkcIz\n" \
"SdhDY2pSS9KP6HBRTdGJaXvHcPaz3BJ023tdS1bTlr8Vd6Gw9KIl8q8ckmcY5fQG\n" \
"BO+QueQA5N06tRn/Arr0PO7gi+s3i+z016zy9vA9r911kTMZHRxAy3QkGSGT2RT+\n" \
"rCpSx4/VBEnkjWNHiDxpg8v+R70rfk/Fla4OndTRQ8Bnc+MUCH7lP59zuDMKz10/\n" \
"NIeWiu5T6CUVAgMBAAGjgbIwga8wDwYDVR0TAQH/BAUwAwEB/zAOBgNVHQ8BAf8E\n" \
"BAMCAQYwbQYIKwYBBQUHAQwEYTBfoV2gWzBZMFcwVRYJaW1hZ2UvZ2lmMCEwHzAH\n" \
"BgUrDgMCGgQUj+XTGoasjY5rw8+AatRIGCx7GS4wJRYjaHR0cDovL2xvZ28udmVy\n" \
"aXNpZ24uY29tL3ZzbG9nby5naWYwHQYDVR0OBBYEFH/TZafC3ey78DAJ80M5+gKv\n" \
"MzEzMA0GCSqGSIb3DQEBBQUAA4IBAQCTJEowX2LP2BqYLz3q3JktvXf2pXkiOOzE\n" \
"p6B4Eq1iDkVwZMXnl2YtmAl+X6/WzChl8gGqCBpH3vn5fJJaCGkgDdk+bW48DW7Y\n" \
"5gaRQBi5+MHt39tBquCWIMnNZBU4gcmU7qKEKQsTb47bDN0lAtukixlE0kF6BWlK\n" \
"WE9gyn6CagsCqiUXObXbf+eEZSqVir2G3l6BFoMtEMze/aiCKm0oHw0LxOXnGiYZ\n" \
"4fQRbxC1lfznQgUy286dUV4otp6F01vvpX1FQHKOtw5rDgb7MzVIcbidJ4vEZV8N\n" \
"hnacRHr2lVz2XTIIM6RUthg/aFzyQkqFOFSDX9HoLPKsEdao7WNq\n" \
"-----END CERTIFICATE-----\n";






WiFiClientSecure client;

// Constructor of Class TCPClientSecure   (function name = class name)
TCPClientSecure::TCPClientSecure()
{
	client.setCACert(rootCABuff);
    client.setCertificate(certificateBuff); 
	client.setPrivateKey(privateKeyBuff);
}



// Closes TCP/IP-connection
// returns 1 when ok, 0 when error

int TCPClientSecure::Close()
{
    int Result = 0;

    client.stop();

    // no detection of error in this implementation of function
    Result = 1;   
    
    return Result;
}




// Establish TCP/IP-connection to TCP/IP-Server
// returns 1 when ok, < 0 when error

int TCPClientSecure::Connect(char* IPAddressString, int Port)
{
    // see https://www.arduino.cc/en/Reference/ClientConnect
    // Returns an int (1,-1,-2,-3,-4) indicating connection status :
    // SUCCESS 1
    // TIMED_OUT -1
    // INVALID_SERVER -2
    // TRUNCATED -3
    // INVALID_RESPONSE -4
    
    int Result = client.connect(IPAddressString, Port);

    return Result;
}



// Polls if bytes were received via TCP/IP from server
// stores it in TCPBytesReceivedBuffer which must be allocated by the calling function
// returns the number of bytes received, 0 when no bytes were received

int TCPClientSecure::ReceiveIfAvailable(byte TCPBytesReceivedBuffer[])
{
    
    // read() read 1 character from a stream into a buffer. 

	int TCPBytesReceivedCount = 0;
    
	while ((client.available() > 0) && (TCPBytesReceivedCount < 120))
	{
		// here we read 120 Bytes max.
		
		
		TCPBytesReceivedBuffer[TCPBytesReceivedCount] = client.read();
		
		TCPBytesReceivedCount++;
	}
	
    // number of bytes received from server
    return TCPBytesReceivedCount;
}





// Polls if bytes were received via TCP/IP from server
// stores it in TCPBytesReceivedBuffer which must be allocated by the calling function
// returns the number of bytes received, 0 when no bytes were received

int TCPClientSecure::Receive(byte TCPBytesReceivedBuffer[])
{
    // see https://www.arduino.cc/en/Reference/StreamReadBytes
    // readBytes() read characters from a stream into a buffer. 
    // The function terminates if the determined length has been read, or it times out (see setTimeout()).

    // here we read 120 Bytes max.
    // Timeout is 1000 ms by default
    int TCPBytesReceivedCount = client.readBytes(TCPBytesReceivedBuffer, 120);

    // number of bytes received from server
    return TCPBytesReceivedCount;
}



// send out bytes via TCP/IP to server
// data contains the bytes to be sent out, len is the number of bytes to be sent out
// returns 1 when ok, 0 when error

int TCPClientSecure::Send(byte data[], int len)
{
     int Result = 0;

     // see https://www.arduino.cc/en/Reference/ClientWrite
     // write() returns the number of bytes written. It is not necessary to read this value.
     int BytesWritten = client.write(data,len);

     // if all bytes were sent out, we assume everything is ok
     if (BytesWritten == len)
     {     
      Result = 1;   
     }

     return Result;
};


